// DE_method.cpp: implementation of the CDE_method class.
//
//////////////////////////////////////////////////////////////////////

#include "DE_method.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDE_method::CDE_method()
{
	pop_size     = population_size;

	/************************************************************************/
	/* default parameter settings for DE method                             */
	/************************************************************************/
	m_F              = 0.5;
	m_CR             = 0.9;
	m_strategy       = 1;		// DE/rand/1/bin

	JADE_mu_CR		= 0.5;
	JADE_mu_FF		= 0.5;


	JADE_c			= 0.1;
	JADE_p			= 0.05;		// 0.05 ~ 0.2
	archive_size	= 0;
}

CDE_method::~CDE_method()
{
}

void CDE_method::init_variables()
{
	if ((CIndividual::index_of_normal) == 0)
	{
		printf("Are you kinding me?\nThere is no function to be optimized.\n");
		exit(0);
	}

	if (CIndividual::index_of_normal != 0)
	{
		init_normal_variables();
	}

	max_interval = 0;
	double temp;
	for (int i=0;i<N_of_x;i++)
	{
		temp = upper_bounds[i]-lower_bounds[i];
		max_interval += temp*temp;
	}
	max_interval = sqrt(max_interval);
}

void CDE_method::init_normal_variables()
{
	int i;
	int n = N_of_x;
	func_flag = CIndividual::index_of_normal;

	switch(func_flag) {
	case 1:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -100.0;
			upper_bounds[i] = 100.0;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 2:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -10.0;
			upper_bounds[i] = 10.0;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 3:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -100.0;
			upper_bounds[i] = 100.0;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 4:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -100.0;
			upper_bounds[i] = 100.0;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 5:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -30.0;
			upper_bounds[i] = 30.0;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 6:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -100.0;
			upper_bounds[i] = 100.0;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 7:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -1.28;
			upper_bounds[i] = 1.28;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 8:// -418.982887272434*D
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -500.0;
			upper_bounds[i] = 500.0;
		}
		known_optimal = 0.0;	// 7.27595761418343e-012
		MINIMIZE = 1;
		break;
	case 9:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -5.12;
			upper_bounds[i] = 5.12;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 10:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -32.0;
			upper_bounds[i] = 32.0;
		}
		known_optimal = 5.88721779659630e-016;
		MINIMIZE = 1;
		break;
	case 11:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -600.0;
			upper_bounds[i] = 600.0;
		}
		known_optimal = 0.0;
		MINIMIZE = 1;
		break;
	case 12:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -50.0;
			upper_bounds[i] = 50.0;
		}
		known_optimal = 1.57044103551778e-032;
		MINIMIZE = 1;
		break;
	case 13:
		for (i=0;i<N_of_x;i++)
		{
			lower_bounds[i] = -50.0;
			upper_bounds[i] = 50.0;
		}
		known_optimal = 1.34969464963993e-032;
		MINIMIZE = 1;
		break;
	default:
		printf("The function you selected does not exist.\n");
		exit(0);
	}
}

void CDE_method::init_pop()
{
	printf("Random based parent_pop initialization method is used.\n");
	init_pop_random();

	// initialize the F and CR of DE
	for (int i=0;i<pop_size;i++)
	{
		parent_pop[i].F        = m_rnd.rndreal(0.1, 1.0);
		parent_pop[i].CR       = m_rnd.rndreal(0.0, 1.0);
		parent_pop[i].accepted = 0;
	}
}

void CDE_method::init_pop_random()
{
	int i, j;
	for (i=0;i<pop_size;i++)
	{
		// for bounded problem
		for (j=0;j<N_of_x;j++)
		{
			parent_pop[i].xreal[j] = m_rnd.rndreal(lower_bounds[j],upper_bounds[j]);
		}
	}
}

void CDE_method::evaluate_ind(CIndividual &indv)
{
	if (CIndividual::index_of_normal != 0)
	{
		evaluate_normal_ind(indv);
	}

	// convert objective function value into fitness
	indv.fitness = MINIMIZE*indv.obj;

	// sum violation of the constrained functions
	if (N_of_constr == 0)
	{
		indv.constr_violation = 0.0;
		indv.feasible = 1;
	}
}

void CDE_method::evaluate_normal_ind(CIndividual &indv)
{
	func_flag = CIndividual::index_of_normal;

	m_func.evaluate_normal_fitness(indv.xreal,indv.obj,indv.constr,
		func_flag, evaluations);
}

void CDE_method::evaluate_pop(CIndividual *pop, int size)
{
	for (int i=0;i<size;i++)
	{
		evaluate_ind(pop[i]);
	}
}

int CDE_method::compare_ind (CIndividual *indv1, CIndividual *indv2)
{
	if((indv1->feasible==TRUE && indv2->feasible==TRUE))
	{
		if(indv1->fitness < indv2->fitness)
		{
			return 1;
		}
		else if (indv1->fitness > indv2->fitness)
		{
			return -1;
		}
		else
		{
			return 0;
		}
	}
	else if(indv1->feasible==TRUE && indv2->feasible==FALSE)
	{
		return 1;
	}
	else if (indv1->feasible==FALSE && indv2->feasible==TRUE)
	{
		return -1;
	}
	else
	{
		if(indv1->constr_violation < indv2->constr_violation)
		{
			return 1;
		}
		else if (indv1->constr_violation > indv2->constr_violation)
		{
			return -1;
		}
		else
		{
			//return 0;
			if (indv1->fitness < indv2->fitness)
			{
				return 1;
			}
			else if (indv1->fitness > indv2->fitness)
			{
				return -1;
			}
			else
			{
				return 0;
			}
		}
	}
}

void CDE_method::find_best_index(CIndividual *pop, int size)
{
	int flag;

	best_index = 0;
	for (int i=1;i<size;i++)
	{
		flag = compare_ind(&pop[i], &pop[best_index]);
		if (flag == 1)
		{
			best_index = i;
		}
	}

	worst_index = 0;
	for (i=1;i<size;i++)
	{
		flag = compare_ind(&pop[i], &pop[worst_index]);
		if (flag == -1)
		{
			worst_index = i;
		}
	}
}

void CDE_method::random_index(int *array_index, int all_size, int size)
{
	int i,j,krand;
	int *a;
	a = new int[all_size];

	for(i = 0;i<all_size;i++)
	{
		a[i] = i;
	}
	for(i=0;i<size;i++)
	{
		j     = m_rnd.rndint(i,(all_size-1));
		krand = a[i];
		a[i]  = a[j];
		a[j]  = krand;
	}	
	for(i=0;i<size;i++)
	{
		array_index[i] = a[i];
	}

	//a = NULL;
	delete []a;
}

void CDE_method::shell_sort_pop(CIndividual *pop, int *list, int size)
{
	int done;
	int step, bound, i, j;
	int temp;

	for (i=0;i<size;i++)
	{
		list[i] = i;
	}

	step = size;  // array length
	while (step > 1) 
	{
		step /= 2;	//halve the step size
		do 
		{
			done   = 1;
			bound  = size - step;
			for (j = 0; j < bound; j++) 
			{
				i = j + step + 1;
				if (compare_ind(&pop[list[j]], &pop[list[i-1]]) == -1) 	
				{
					temp      = list[i-1];
					list[i-1] = list[j];
					list[j]   = temp;
					done      = 0; // if a swap has been made we are not finished yet
				}  // if
			}  // for
		} while (done == 0);   // while
	} //while (step > 1)
}

void CDE_method::shell_sort_array(double *array, int *list, int size)
{
	int done;
	int step, bound, i, j;
	int temp;

	for (i=0;i<size;i++)
	{
		list[i] = i;
	}

	step = size;  // array length
	while (step > 1) 
	{
		step /= 2;	//halve the step size
		do 
		{
			done   = 1;
			bound  = size - step;
			for (j = 0; j < bound; j++) 
			{
				i = j + step + 1;
				if (array[list[j]] > array[list[i-1]]) 	
				{
					temp      = list[i-1];
					list[i-1] = list[j];
					list[j]   = temp;
					done      = 0; // if a swap has been made we are not finished yet
				}  // if
			}  // for
		} while (done == 0);   // while
	} //while (step > 1)
}

void CDE_method::shell_sort_array_t(tFitness *array, int *list, int size)
{
	int done;
	int step, bound, i, j;
	int temp;

	for (i=0;i<size;i++)
	{
		list[i] = i;
	}

	step = size;  // array length
	while (step > 1) 
	{
		step /= 2;	//halve the step size
		do 
		{
			done   = 1;
			bound  = size - step;
			for (j = 0; j < bound; j++) 
			{
				i = j + step + 1;
				if (array[list[j]] > array[list[i-1]]) 	
				{
					temp      = list[i-1];
					list[i-1] = list[j];
					list[j]   = temp;
					done      = 0; // if a swap has been made we are not finished yet
				}  // if
			}  // for
		} while (done == 0);   // while
	} //while (step > 1)
}

double CDE_method::median(double *array, int size)
{
	double value = 0.0;

	int list[2000];
	for (int i=0;i<2000;i++)
	{
		list[i] = i;
	}
	shell_sort_array(array, list, size);
	
	if (size%2 == 1)
	{// the size of the array is odd.
		int media_index;
		media_index = size/2;
		value = array[list[media_index]];
	}
	else
	{// the size of the array is even.
		int a = (size-1)/2;
		int b = size/2;
		value = (array[list[a]]+array[list[b]])/2.0;
	}

	return value;
}

double CDE_method::mean_std(double *array, int size, double &stdv)
{
	double value = 0.0;

	assert(size > 0);

	for (int i=0;i<size;i++)
	{
		value += array[i];
	}
	value = value/((double)size);

	stdv = 0.0;
	for (i=0;i<size;i++)
	{
		stdv += (array[i]-value)*(array[i]-value);
	}
	stdv = sqrt(stdv/((double)size-1));

	return value;
}

tFitness CDE_method::mean_std_t(tFitness *array, int size, tFitness &stdv)
{
	tFitness value = 0.0;

	assert(size > 0);

	for (int i=0;i<size;i++)
	{
		value += array[i];
	}
	value = value/((double)size);

	stdv = 0.0;
	for (i=0;i<size;i++)
	{
		stdv += (array[i]-value)*(array[i]-value);
	}
	stdv = sqrt(stdv/((double)size-1));

	return value;
}

void CDE_method::display_result(long int gen)
{
	if(gen%10==0 || gen == 1)
	{
		//ǰøĻ
		cout<<setw(5)<<gen;
		cout<<setw(8)<<evaluations;
		//ʾԼֵ
		if (N_of_constr != 0)
		{			
			cout<<setw(15)<<best_individual.constr_violation;
		}
		cout.precision(10);					//
		cout<<setw(20)<<MINIMIZE * parent_pop[best_index].fitness;
		cout<<setw(20)<<MINIMIZE * parent_pop[m_rnd.rndint(0, pop_size-1)].fitness;
		cout<<setw(20)<<MINIMIZE * parent_pop[worst_index].fitness<<endl;
	}
}

void CDE_method::report_result(long int gen, ofstream &file)
{
	if (gen < 5 )
	{
		//ǰøļ
		file<<setw(5)<<gen;
		file<<setw(10)<<evaluations;
		file.precision(20);			//
		file<<setw(30)<<MINIMIZE * best_individual.fitness;
		file<<setw(30)<<best_individual.constr_violation;
		//file<<setw(30)<<best_individual.CR;
		file<<endl;
	}
	else
	{
		if ((gen % (max_iterations/out_internal) == 0 ||
			gen >= max_iterations ||
			evaluations >= max_NFFEs) )
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<MINIMIZE * best_individual.fitness;
			file<<setw(30)<<best_individual.constr_violation;
			//file<<setw(30)<<best_individual.CR;
			file<<endl;
		}
	}
}

void CDE_method::report_diversity(long int gen, ofstream &file)
{
	// calculate the diversity of the population
	double x_average[2000];
	for (int i=0;i<2000;i++)
	{
		x_average[i] = 0.0;
	}

	for (i=0;i<N_of_x;i++)
	{
		for (int j=0;j<pop_size;j++)
		{
			x_average[i] += parent_pop[j].xreal[i];
		}
		x_average[i] = x_average[i]/((double)pop_size);
	}

	double diversity = 0.0;
	for (i=0;i<pop_size;i++)
	{
		double temp = 0.0;
		double x;
		for (int j=0;j<N_of_x;j++)
		{
			x = parent_pop[i].xreal[j];
			temp += (x-x_average[j])*(x-x_average[j]);
		}
		diversity += sqrt(temp);
	}
	diversity = diversity/(((double)pop_size)*max_interval);

	pop_diversity = diversity;
	if (gen==1)	pop_diversity0 = pop_diversity;
	
	if (gen < 100 )
	{
		//ǰøļ
		file<<setw(5)<<gen;
		file<<setw(10)<<evaluations;
		file.precision(20);			//
		file<<setw(30)<<diversity<<endl;
	}
	else
	{
		if ((gen % out_internal == 0 ||
			gen >= max_iterations ||
			evaluations >= max_NFFEs) )
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<diversity<<endl;
		}
	}
}

void CDE_method::report_parameter(long int gen, ofstream &file)
{	
	int counter = max_iterations/20;
	if (method_flag==5 || method_flag==6 )
	{
		if (gen <=10 || gen%counter == 0)
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<JADE_mu_CR;
			file<<setw(30)<<JADE_mu_FF;
			file<<setw(30)<<"0";
			file<<endl;
		}
	}
	else
	{
		if (gen <=10 || gen%counter == 0)
		{
			//ǰøļ
			file<<setw(5)<<gen;
			file<<setw(10)<<evaluations;
			file.precision(20);			//
			file<<setw(30)<<best_individual.CR;
			file<<setw(30)<<best_individual.F;
			file<<setw(30)<<"0";
			file<<endl;
		}
	}
}

void CDE_method::sort_pop(CIndividual *pop, int size)
{
	for (int i=0;i<size;i++)
	{
		sort_index[i] = i;
	}
	shell_sort_pop(pop, sort_index, size);
}

// copy files from the source directory to the target directory
void CDE_method::copy_files(char *source, char *target)
{
	FILE *fs,*ft;
	char ma[204];
	unsigned int rd=0;

	fs=fopen(source, "rb");
	ft=fopen(target, "wb"); 
	if (fs&&ft) 
	{
		while (!feof(fs))
		{
			rd=fread(ma,sizeof(char),204,fs);
			fwrite(ma,sizeof(char),rd,ft);
		}
		fclose(fs);
		fclose(ft);
	}
}

long CDE_method::filesize(FILE *stream)
{
	long curpos, length;
	curpos = ftell(stream);
	fseek(stream, 0L, SEEK_END);
	length = ftell(stream);
	fseek(stream, curpos, SEEK_SET);
	return length;
}

int CDE_method::copyfile(const char* src,const char* dest)
{
	FILE *fp1,*fp2;
	int fsize,factread;
	static unsigned char buffer[SIZEOFBUFFER];

	fp1=fopen(src,  "rb");
	fp2=fopen(dest, "wb+");
	if (!fp1 || !fp2) 
	{
		return 0;
	}

    for (fsize=filesize(fp1);fsize>0;fsize-=SIZEOFBUFFER)
    {
		factread=fread(buffer,1,SIZEOFBUFFER,fp1);
		fwrite(buffer,factread,1,fp2);
    }
	fclose(fp1);
	fclose(fp2);
	return 1;
}

// create folders, only for batch execution
void CDE_method::create_folders(int method_ii, int strategy_ii)
{
	int status;
	int func_index;
	if (CIndividual::index_of_normal)	func_index = CIndividual::index_of_normal;

	/************************************************************************/
	/* Create the zero-order folders: function level                         */
	/************************************************************************/
	char f_name_0[150];
	// set the folder name
	sprintf(f_name_0,"%s%s%d", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index);

	// create the folders
	status = mkdir(f_name_0);
	//if (!status==0)	printf("Unable to create directory 1\n");
	/************************************************************************/

	/************************************************************************/
	/* Create the first-order folders: method level                         */
	/************************************************************************/
	char f_name0[150];
	// set the folder name
	sprintf(f_name0,"%s%s%d%s%d", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii);

	// create the folders
	status = mkdir(f_name0);
	//if (!status==0)	printf("Unable to create directory 1\n");
	/************************************************************************/

	/************************************************************************/
	/* Create the second-order folders: strategy level                      */
	/************************************************************************/
	char f_name00[150];
	// set the folder name
	sprintf(f_name00,"%s%s%d%s%d%s%d", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii);

	// create the folders
	status = mkdir(f_name00);
	//if (!status==0)	printf("Unable to create directory 1\n");
	/************************************************************************/

	/************************************************************************/
	/* Copy the statistic file to the strategy_level folder                 */
	/************************************************************************/
	char source_file_name[150] = ".\\Results\\1-DA-50.exe";
	char target_file_name[150];
	sprintf(target_file_name, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\1-DA-50.exe");
	//copy_files(source_file_name, target_file_name);
	copyfile(source_file_name, target_file_name);

	/************************************************************************/
	/* Create the third-order folders: results level                        */
	/************************************************************************/
	char f_name1[150];
	char f_name2[150];
	char f_name3[150];
	// set the folder name
	sprintf(f_name1, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\process");
	sprintf(f_name2, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\diversity");
	sprintf(f_name3, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? "\\f0" : "\\f"), func_index, "\\algorithm_", method_ii, "\\strategy_", strategy_ii, "\\migration");

	// create the folders
	status = mkdir(f_name1);
	//if (!status==0)	printf("Unable to create directory 2\n");
	status = mkdir(f_name2);
	//if (!status==0)	printf("Unable to create directory 3\n");
	status = mkdir(f_name3);
	//if (!status==0)	printf("Unable to create directory 4\n");
	/************************************************************************/
}

void CDE_method::Run_Optimizer(int func_flag, int run_no, double seed, int method, int strategy_ii)
{
	if (func_flag>=1 && func_flag<=50)
	{
		CIndividual::index_of_normal  = func_flag;
		CIndividual::index_of_cec2013 = 0;

		CIndividual::Max_of_NFEs = 100000;
		if (func_flag>=34 && func_flag<=43)	CIndividual::Max_of_NFEs = 10000;
		
		max_iterations = CIndividual::Max_of_NFEs/population_size;
		max_NFFEs      = CIndividual::Max_of_NFEs;
	}
	else
	{
		printf("The function to be optimized is not defined!\n");
		exit(0);
	}

	m_strategy   = strategy_ii;
	radius_index = strategy_ii;

	if (batch_execution==1)
	{
		create_folders(method, strategy_ii);
	}
	
	switch(method) {
	case 1:
		printf("Algorithm: CoDE_method.\n");
		break;
	case 2:
		printf("Algorithm: NoDE_CoDE.\n");
		break;		
	case 3:
		printf("Algorithm: JADE_method.\n");
		break;	
	case 4:
		printf("Algorithm: NoDE_JADE.\n");
		break;
	default:
		printf("The method does not exist!\n");
		exit(0);
	}

	int i = 0; 
	
	method_flag = method;
	rnd_seed    = seed;

	int func_index;
	if (CIndividual::index_of_normal)	func_index = CIndividual::index_of_normal;

	ofstream SummaryFile;
	char     f_name0[150];
	if (batch_execution==0)	
	{
		SummaryFile.open(".\\results\\summary.txt",ios::app);
	}
	if (batch_execution==1) 
	{
		sprintf(f_name0, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\summary.txt");
		SummaryFile.open(f_name0, ios::app);
	}

	char f_name1[150];
	if (batch_execution==0)	
	{
		sprintf(f_name1,"%s%d%s",".\\results\\process\\process_",run_no+1,".txt");	
	}
	if (batch_execution==1) 
	{
		sprintf(f_name1, "%s%s%d%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\process/process_" ,run_no+1,".txt");
	}
	ofstream file_process(f_name1);

	char f_name2[150];
	if (batch_execution==0)	
	{
		sprintf(f_name2,"%s%d%s",".\\results\\diversity\\diversity_",run_no+1,".txt");	
	}
	if (batch_execution==1) 
	{
		sprintf(f_name2, "%s%s%d%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\diversity/diversity_",run_no+1,".txt");
	}
	ofstream file_diversity(f_name2);

	char f_name3[150];
	if (batch_execution==0)	
	{
		sprintf(f_name3,"%s%d%s",".\\results\\migration\\migration_",run_no+1,".txt");	
	}
	if (batch_execution==1) 
	{
		sprintf(f_name3, "%s%s%d%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\migration/migration_",run_no+1,".txt");
	}
	ofstream file_migration(f_name3);

	clock_t start, finish;
	double time_consume;

	time_consume = 0.0;
	start = clock();						// starts the clock
	
	srand((unsigned)time(0));
	m_rnd.randomize(seed);

	evaluations = 0;						// reset the NFFEs

	init_variables();	
	init_pop();
	evaluate_pop(parent_pop, pop_size);
	find_best_index(parent_pop, pop_size);
	best_individual = parent_pop[best_index];
	m_strategy = strategy_ii;
	gen = 1;								// current generation number
	int counter=0;							// only for restarting the population

	report_result(gen, file_process);
	report_diversity(gen, file_diversity);
	report_parameter(gen, file_migration);

	int feasible_flag = 0;					// check to get the feasible individual first
	flag_precision = 0;						// check to arrive the required value
	/* -------- add different optimizer here -------- */
	while ( (evaluations < max_NFFEs) && (gen < max_iterations) )
	{
		finish = clock();					// time consuming of this generation
		time_consume = (double)(finish-start)/CLOCKS_PER_SEC;

		// report the results in the screen
		/* comment the following routines to save the time_consuming */
		//display_result(gen);
		
		// ADD YOUR OPTIMIZER HERE
		switch(method) {
		case 1:
			run_CoDE_method();
			break;
		case 2:
			run_NoDE_CoDE();
			break;
		case 3:
			run_JADE_method();
			break;
		case 4:
			run_NoDE_JADE();
			break;
		default:
			printf("The method selected does not exist.\n");
			exit(0);
		}

		gen++;

		find_best_index(parent_pop, pop_size);
		
		if (compare_ind(&parent_pop[best_index], &best_individual) != -1)
		{
			best_individual = parent_pop[best_index];
		}
		
		report_result(gen, file_process);
		report_diversity(gen, file_diversity);
		report_parameter(gen, file_migration);

		if (flag_precision == 0 && 
			fabs(MINIMIZE*best_individual.fitness - known_optimal) < PRECISION
			&& best_individual.feasible == 1)
		{
			flag_precision = 1;
			ofstream SummaryFile1;
			char     f_name00[150];
			if (batch_execution==0)	
			{
				SummaryFile1.open(".\\results\\evaluations.txt",ios::app);	
			}
			if (batch_execution==1) 
			{
				sprintf(f_name00, "%s%s%d%s%d%s%d%s", ".\\results", (func_index<10 ? ".\\f0" : ".\\f"), func_index, ".\\algorithm_", method, ".\\strategy_", strategy_ii, ".\\evaluations.txt");
				SummaryFile1.open(f_name00, ios::app);
			}			
			SummaryFile1<<setw(9)<<evaluations<<endl;
			SummaryFile1.close();
			//break;		// һľͳӦֵ۴ʱʹ
		}
		if (N_of_constr != 0 && feasible_flag == 0 && best_individual.feasible == 1)
		{
			feasible_flag = 1;
			ofstream SummaryFile1;
			SummaryFile1.open(".\\results\\feasible_NFFEs.txt",ios::app);
			SummaryFile1<<setw(9)<<evaluations<<endl;
			SummaryFile1.close();
		}
	}
	/* -------- add different optimizer here -------- */
	
	printf("The total running time is %f s.\n", time_consume);
	if (CIndividual::index_of_normal != 0)
	{
		printf("The routine in optimizing f(%d) exits successfully.\n\n", CIndividual::index_of_normal);
	}

	SummaryFile.precision(15);
	SummaryFile<<setw(5)<<gen<<setw(9)<<evaluations
		<<setw(25)<<MINIMIZE * best_individual.fitness
		<<setw(8)<<time_consume
		<<setw(25)<<best_individual.constr_violation<<endl;

	file_process.close();
	file_diversity.close();
	file_migration.close();
	SummaryFile.close();
	
	//return;
}


/************************************************************************/
/* For the CoDE method                                                  */
/************************************************************************/
void CDE_method::run_CoDE_method()
{
	//assert(pop_size==30);		// NP=30 is used in CoDE

	int		i, j, para_index;
	double	low, up;

	// initialize the parameter pool
	if (gen == 1)
	{
		double tt_parameters[3][2] = {0.1, 1.0, 0.9, 1.0, 0.2, 0.8};
		for (i=0; i<3; i++)
		{
			for (j=0; j<2; j++)
			{
				CoDE_CR_F[i][j] = tt_parameters[i][j];
			}
		}
	}

	// Generate the new trail vectors
	int r1, r2, r3, r4, r5;
	int j_rand;
	for (i=0;i<pop_size;i++)
	{
		/************************************************************************/
		/* generate the first individual with DE/rand/1/bin                     */
		/************************************************************************/
		// select three different indices
		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i || r3 == r2 || r3 == r1);

		// randomly select the CR and F values
		para_index = m_rnd.rndint(1, 3);
		m_CR = CoDE_CR_F[para_index-1][0];
		m_F  = CoDE_CR_F[para_index-1][1];

		// generate the first individual
		j_rand = m_rnd.rndint(0, N_of_x-1);
		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];

			if (m_rnd.rndreal(0,1)<m_CR || j==j_rand)
			{
				// DE/rand/1/bin
				CoDE_indv[0].xreal[j] = parent_pop[r1].xreal[j]
					+ m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);

				if (CoDE_indv[0].xreal[j] < low || CoDE_indv[0].xreal[j] > up)
				{
					CoDE_indv[0].xreal[j] = m_rnd.rndreal(low, up);
				}
			}
			else
			{
				CoDE_indv[0].xreal[j] = parent_pop[i].xreal[j];
			}
		}

		evaluate_ind(CoDE_indv[0]);
		child_pop[i] = CoDE_indv[0];
	
		/************************************************************************/
		/* generate the second individual with DE/rand/2/bin                    */
		/************************************************************************/
		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i || r3 == r2 || r3 == r1);
		do { 
			r4 = m_rnd.rndint(0, pop_size-1);
		} while(r4 == i || r4 == r3 || r4 == r2 || r4 == r1);
		do { 
			r5 = m_rnd.rndint(0, pop_size-1);
		} while(r5 == i || r5 == r4 || r5 == r3 || r5 == r2 || r5 == r1);

		para_index = m_rnd.rndint(1, 3);
		m_CR = CoDE_CR_F[para_index-1][0];
		m_F  = CoDE_CR_F[para_index-1][1];

		j_rand = m_rnd.rndint(0, N_of_x-1);
		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];

			if (m_rnd.rndreal(0,1)<m_CR || j==j_rand)
			{// DE/rand/2/bin
				CoDE_indv[1].xreal[j] = parent_pop[r1].xreal[j]
					+ m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j])
					+ m_F*(parent_pop[r4].xreal[j]-parent_pop[r5].xreal[j]);

				if (CoDE_indv[1].xreal[j] < low || CoDE_indv[1].xreal[j] > up)
				{
					CoDE_indv[1].xreal[j] = m_rnd.rndreal(low, up);
				}
			}
			else
			{
				CoDE_indv[1].xreal[j] = parent_pop[i].xreal[j];
			}
		}

		evaluate_ind(CoDE_indv[1]);
		if (compare_ind(&CoDE_indv[1], &child_pop[i]) != -1)
		{
			child_pop[i] = CoDE_indv[1];
		}

		/************************************************************************/
		/* generate the third individual with DE/current-to-rand/1              */
		/************************************************************************/
		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i);// || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i);// || r3 == r2 || r3 == r1);

		para_index = m_rnd.rndint(1, 3);
		m_CR = CoDE_CR_F[para_index-1][0];
		m_F  = CoDE_CR_F[para_index-1][1];

		double KK = m_rnd.rndreal(0.0, 1.0);
		for (j=0;j<N_of_x;j++)
		{// DE/current-to-rand/1
			CoDE_indv[2].xreal[j]=parent_pop[i].xreal[j]
				+(KK*(parent_pop[r1].xreal[j]-parent_pop[i].xreal[j]))
				+(m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]));

			low=lower_bounds[j];
			up =upper_bounds[j];

			if( (CoDE_indv[2].xreal[j]>up) || (CoDE_indv[2].xreal[j]<low) )
			{
				CoDE_indv[2].xreal[j]=m_rnd.rndreal(low, up);
			}
		}

		evaluate_ind(CoDE_indv[2]);
		if (compare_ind(&CoDE_indv[2], &child_pop[i]) != -1)
		{
			child_pop[i] = CoDE_indv[2];
		}
	}

	// Selection 
	for (i=0;i<pop_size;i++)
	{
		int flag = compare_ind(&child_pop[i], &parent_pop[i]);
		if (flag != -1)
		{
			parent_pop[i] = child_pop[i];
		}
	}
}

/************************************************************************/
/* For the nonparametric density estimation                             */
/************************************************************************/
double CDE_method::Euc_dist(double *x, double *y, int size)
{
	double rst = -1.0;

	rst = 0.0;
	int    i;
	for (i=0; i<size; i++)
	{
		rst += (x[i] - y[i]) * (x[i] - y[i]);
	}
	rst = sqrt(rst/(double)size);

	return rst;
}

double CDE_method::infinity_norm_dist(double *x, double *y, int size)
{
	double rst = -1.0;

	double d[1000];		assert(size<=1000);
	int    i;
	for (i=0; i<size; i++)
	{
		d[i] = x[i] - y[i];
		d[i] = (d[i]>0) ? d[i] : -d[i];
	}

	rst = d[0];
	for (i=0; i<size; i++)
	{
		if (d[i] > rst)
		{
			rst = d[i];
		}
	}

	return rst;
}

double CDE_method::NoDE_Gaussian_kernel(double u)
{
	double rst = -1.0;

	rst = 1.0/sqrt(2.0*PI) * exp(-u*u/2.0);

	return rst;
}

int CDE_method::update_tmp_boundary(const double x, const int j)
{
	int flag = 0;
	if (x < tmp_lower_bounder[j])
	{
		flag                 = 1;
		tmp_lower_bounder[j] = x;
	}
	if (x > tmp_upper_bounder[j])
	{
		flag                 = 1;
		tmp_upper_bounder[j] = x;
	}

	return flag;
}

double CDE_method::estimate_current_window_size(CIndividual *pop, int size)
{
	double rst = -1.0;
	
	int    n = N_of_x;
	assert(n <= 2000);

	int i, j;

	/* find the lower and upper bounds of each variable in the current population */
	for (j=0; j<n; j++)
	{
		tmp_lower_bounder[j] = pop[0].xreal[j];
		for (i=0; i<size; i++)
		{
			if (pop[i].xreal[j] < tmp_lower_bounder[j])
			{
				tmp_lower_bounder[j] = pop[i].xreal[j];
			}
		}

		tmp_upper_bounder[j] = pop[0].xreal[j];
		for (i=0; i<size; i++)
		{
			if (pop[i].xreal[j] > tmp_upper_bounder[j])
			{
				tmp_upper_bounder[j] = pop[i].xreal[j];
			}
		}
	}

	/* calculate the window size of the current population */
	double temp = 0.0;
	for (j=0; j<n; j++)
	{
		temp      += (tmp_upper_bounder[j] - tmp_lower_bounder[j])*(tmp_upper_bounder[j] - tmp_lower_bounder[j]);
		NoDE_w[j] = (tmp_upper_bounder[j] - tmp_lower_bounder[j]);
	}
	rst = sqrt(temp/(double)n);

	return rst;
}

void CDE_method::calculate_ranking_UOPs(CIndividual *pop, int size, int *ranks)
{
	/* sort the population in descending order based on the fitness */
	shell_sort_pop(pop, sort_index, size);

	/* calculate the ranking of each solution */
	int i;
	for (i=0; i<size; i++)
	{
		ranks[sort_index[i]] = size - i;			// to make sure the worst solution have non-zero ranking
	}
}

void CDE_method::calculate_weighted_center(CIndividual *pop, int size)
{
	int	   i, j;
	double ranks = 0;
	for (i=0;i<pop_size;i++)
	{
		ranks += (double)NoDE_ranks[i];
	}
	double temp = 0.0;
	for (j=0;j<N_of_x;j++)
	{
		temp = 0.0;
		for (i=0;i<size;i++)
		{
			temp += NoDE_ranks[i]/ranks * pop[i].xreal[j];
		}
		weighted_center[j] = temp;
		assert(temp>=lower_bounds[j] && temp<=upper_bounds[j]);
	}
}

double CDE_method::NDE_of_solution(int baseindex, CIndividual *pop, int size, double *xreal)
{
	double rst = -1.0;
	int    i   = 0, j = 0;

	/* calculate the window size of the current population */	
	for (j=0; j<N_of_x; j++)
	{
		NoDE_w[j] = (tmp_upper_bounder[j] - tmp_lower_bounder[j]);	
	}

	double tmp = 0.0;
	tmp = Euc_dist(tmp_upper_bounder, tmp_lower_bounder, N_of_x);

	double w;				// window size
	w = tmp;
	assert(w >= 0.);
	if (w == 0.0)	return 0;

	double temp = 0.0, dist, u;

	/************************************************************************/
	/* global version                                                       */
	/************************************************************************/
	assert(radius_index>=0 && radius_index<=5);
	/* common method */
	temp = 0.0;
	for (i=0; i<size; i++)
	{
		dist = Euc_dist(xreal, pop[i].xreal, N_of_x);
		u    = dist/w;
		temp += (double)NoDE_ranks[i] * (1.0-u*u);		
	}
		
	rst = temp/(double)size;

	return rst;
}

int CDE_method::offspring_fitering(int baseindex, CIndividual *pop, int size, CIndividual &child)
{
	int		i, i_max, j;
	double	prob[100], max_p=0;

	assert(candidate_size > 1);

	/* calculate the probability by NDE for each candidate */
	for (i=0; i<candidate_size ;i++)
	{
		prob[i] = NDE_of_solution(baseindex, pop, size, candidate_x[i]);
		assert(prob[i]>=0.0);
	}

	/************************************************************************/
	/* find the maximal probability */
	max_p = prob[0];
	i_max = 0;
	for (i=0; i<candidate_size ;i++)
	{
		if (prob[i] > max_p)
		{
			max_p = prob[i];
			i_max = i;
		}
	}

	/* tournament selection 
	int r1, r2, r3;
	r1 = m_rnd.rndint(0, candidate_size-1);
	do {
		r2 = m_rnd.rndint(0, candidate_size-1);
	} while(r1 == r2);
	r3 = (prob[r1] > prob[r2]) ? r1 : r2;
	i_max = r3;*/
	/************************************************************************/

	for (j=0; j<N_of_x; j++)
	{
		child.xreal[j] = candidate_x[i_max][j];
	}

	return (i_max+1);
}



void CDE_method::run_NoDE_CoDE()
{
	//assert(pop_size==30);		// NP=30 is used in CoDE

	int		i, j, para_index[3];
	double	low, up;

	// initialize the parameter pool
	if (gen == 1)
	{
		double tt_parameters[3][2] = {0.1, 1.0, 0.9, 1.0, 0.2, 0.8};
		for (i=0; i<3; i++)
		{
			for (j=0; j<2; j++)
			{
				CoDE_CR_F[i][j] = tt_parameters[i][j];
			}
		}
	}

	/************************************************************************/
	/* Calculate the window size and rankings in NoDE                       */
	/************************************************************************/
	candidate_size   = 3;
	NoDE_window_size = estimate_current_window_size(parent_pop, pop_size);
	calculate_ranking_UOPs(parent_pop, pop_size, NoDE_ranks);

	// Generate the new trail vectors
	int r1, r2, r3, r4, r5;
	int j_rand;
	for (i=0;i<pop_size;i++)
	{
		/************************************************************************/
		/* generate the first individual with DE/rand/1/bin                     */
		/************************************************************************/
		// select three different indices
		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i || r3 == r2 || r3 == r1);

		// randomly select the CR and F values
		para_index[0] = m_rnd.rndint(1, 3);	assert(para_index[0]>=1 && para_index[0]<=3);
		m_CR          = CoDE_CR_F[para_index[0]-1][0];
		m_F           = CoDE_CR_F[para_index[0]-1][1];

		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];

			CoDE_indv[0].xreal[j] = parent_pop[r1].xreal[j]
				+ m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);

			if (CoDE_indv[0].xreal[j] < low || CoDE_indv[0].xreal[j] > up)
			{
				CoDE_indv[0].xreal[j] = m_rnd.rndreal(low, up);
			}

			candidate_x[0][j] = CoDE_indv[0].xreal[j];

			update_tmp_boundary(candidate_x[0][j], j);
		}
	
		/************************************************************************/
		/* generate the second individual with DE/rand/2/bin                    */
		/************************************************************************/
		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i || r3 == r2 || r3 == r1);
		do { 
			r4 = m_rnd.rndint(0, pop_size-1);
		} while(r4 == i || r4 == r3 || r4 == r2 || r4 == r1);
		do { 
			r5 = m_rnd.rndint(0, pop_size-1);
		} while(r5 == i || r5 == r4 || r5 == r3 || r5 == r2 || r5 == r1);

		para_index[1] = m_rnd.rndint(1, 3);
		m_CR          = CoDE_CR_F[para_index[1]-1][0];
		m_F           = CoDE_CR_F[para_index[1]-1][1];

		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];

			CoDE_indv[1].xreal[j] = parent_pop[r1].xreal[j]
				+ m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j])
				+ m_F*(parent_pop[r4].xreal[j]-parent_pop[r5].xreal[j]);

			if (CoDE_indv[1].xreal[j] < low || CoDE_indv[1].xreal[j] > up)
			{
				CoDE_indv[1].xreal[j] = m_rnd.rndreal(low, up);
			}

			candidate_x[1][j] = CoDE_indv[1].xreal[j];

			update_tmp_boundary(candidate_x[1][j], j);
		}

		/************************************************************************/
		/* generate the third individual with DE/current-to-rand/1              */
		/************************************************************************/
		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i);// || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i);// || r3 == r2 || r3 == r1);

		para_index[2] = m_rnd.rndint(1, 3);
		m_CR          = CoDE_CR_F[para_index[2]-1][0];
		m_F           = CoDE_CR_F[para_index[2]-1][1];

		double KKK = m_rnd.rndreal(0.0, 1.0);
		for (j=0;j<N_of_x;j++)
		{// DE/current-to-rand/1
			CoDE_indv[2].xreal[j]=parent_pop[i].xreal[j]
				+(KKK*(parent_pop[r1].xreal[j]-parent_pop[i].xreal[j]))
				+(m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]));

			low=lower_bounds[j];
			up =upper_bounds[j];

			if( (CoDE_indv[2].xreal[j]>up) || (CoDE_indv[2].xreal[j]<low) )
			{
				CoDE_indv[2].xreal[j]=m_rnd.rndreal(low, up);
			}

			candidate_x[2][j] = CoDE_indv[2].xreal[j];

			update_tmp_boundary(candidate_x[2][j], j);
		}

		/************************************************************************/
		/* Offspring filtering with the NoDE method                             */
		/************************************************************************/
		int i_max = offspring_fitering(i, parent_pop, pop_size, child_pop[i]);

		/* crossover */
		m_CR   = CoDE_CR_F[i_max-1][0];
		j_rand = m_rnd.rndint(0, N_of_x-1);
		for (j=0;j<N_of_x;j++)
		{
			if (i_max != para_index[2])
			{			
				if (m_rnd.rndreal(0,1)<m_CR || j==j_rand)
				{
					child_pop[i].xreal[j] = child_pop[i].xreal[j];
				}
				else
				{
					child_pop[i].xreal[j] = parent_pop[i].xreal[j];
				}
			}
		}
	}

	evaluate_pop(child_pop, pop_size);

	// Selection 
	for (i=0;i<pop_size;i++)
	{
		int flag = compare_ind(&child_pop[i], &parent_pop[i]);
		if (flag != -1)
		{
			parent_pop[i] = child_pop[i];
		}
	}
}

/************************************************************************/
/* For the JADE methods                                                 */
/************************************************************************/
void CDE_method::run_JADE_method()
{
	int    i, j;
	int    r1, r2, r3;
	double low, up;

	double FF[population_size];
	double CR[population_size];
	int    CR_i[population_size][N_of_x];

	/* Step 1: Initialize the JADE-related parameters */
	int    archive_index;	// if r3 is chosen from archive archive_index=1, else =0
	int    p_index = -1;
	if (gen == 1)
	{
		JADE_mu_CR  = 0.5;
		JADE_mu_FF  = 0.5;
		JADE_c      = 0.1;

		JADE_p       = 0.05;
		archive_size = 0;
	}
	/*------------------------------------------------*/

	sort_pop(parent_pop, pop_size);

	/* Generate the CR and F values */
	for (i=0;i<pop_size;i++)
	{
		// generate F value for each individual (individual level)
		do {
			FF[i] = m_rnd.cauchy(JADE_mu_FF, 0.1);
		} while (FF[i] <= 0.0);
		if (FF[i] > 1.0)	FF[i] = 1.0;

		// generate CR value for each dimension of each individual (component level)
		CR[i] = m_rnd.gaussian(JADE_mu_CR, 0.1);
		if (CR[i] < 0.0)	CR[i] = 0.0;
		if (CR[i] > 1.0)	CR[i] = 1.0;
	}

	for (i=0;i<pop_size;i++)
	{
		// randomly choose the p_best individual
		p_index = m_rnd.rndint(0, int(JADE_p*pop_size)-1);
		p_index = sort_index[p_index];

		// select five parents randomly
		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i || r3 == r2 || r3 == r1);

		/************************************************************************/
		/* For JADE strategies with archive                                     */
		/************************************************************************/
		if (m_strategy==3 || m_strategy==4)
		{
			// choose r3 from the union of parent population and the archive
			archive_index = 0;
			int k = pop_size+archive_size;
			r3 = m_rnd.rndint(0, k-1);
			if (r3>=0 && r3<pop_size)
			{// r3 is chosen from the parent population
				while(r3 == r2 || r3 == r1 || r3 == i)
				{
					r3 = m_rnd.rndint(0, pop_size-1);
				}
			}
			else
			{// r3 is chosen from the archive
				archive_index = 1;
				r3 = r3-pop_size;
			}
		}
		/************************************************************************/

		m_F = FF[i];

		int j_rand = m_rnd.rndint(0, N_of_x-1);
		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];

			if (m_rnd.rndreal(0,1)<CR[i] || j==j_rand)
			{
				switch(m_strategy) {
				case 1:// DE/current-to-pbest mutation
					child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
						m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
						m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					break;
				case 2:// DE/rand-to-pbest mutation
					child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
						m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
						m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					break;
				case 3:// DE/current-to-pbest mutation with archive
					if (archive_index==0)
					{
						child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					}
					else
					{
						child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-archive_pop[r3].xreal[j]);
					}
					break;
				case 4:// DE/rand-to-pbest mutation with archive
					if (archive_index==0)
					{
						child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
					}
					else
					{
						child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
							m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
							m_F*(parent_pop[r2].xreal[j]-archive_pop[r3].xreal[j]);
					}
					break;
				default:
					printf("The scheme in DE method does not exist.\n");
					exit(0);
				}

				if (child_pop[i].xreal[j] < low || child_pop[i].xreal[j] > up)
				{
					child_pop[i].xreal[j] = m_rnd.rndreal(low, up);
				}
			}
			else
			{
				child_pop[i].xreal[j] = parent_pop[i].xreal[j];
			}
		}
	}

	evaluate_pop(child_pop, pop_size);

	double SS_FF[population_size], tt_CR[population_size];
	int    SS_CR[population_size][N_of_x];
	tFitness SS_reward[population_size], rr;
	int    SS_size = 0;						// The successful size of S_CR and S_F
	for (i=0;i<pop_size;i++)
	{
		int flag = compare_ind(&child_pop[i], &parent_pop[i]);
		if (flag != -1)
		{
			/************************************************************************/
			/* Calculate the relative fitness improvement                           */
			/************************************************************************/
			// get the absolute fitness improvement
			rr = parent_pop[i].fitness - child_pop[i].fitness; // for minimization
			if(parent_pop[i].fitness>=DBL_MAX)
			{
				printf("ERRRRRRRRRRRRRRRRRRRRRRRR in DBL_MAX\n");
				exit(0);
			}

			// calculate the relative fitness improvement
			if (child_pop[i].fitness > 0.0)
			{
				rr = (best_individual.fitness*rr)/child_pop[i].fitness;
			}
			/************************************************************************/

			/* Step 3: Update the accepting flag and store the successful CR and F values */
			for (j=0;j<N_of_x;j++)
			{
				SS_CR[SS_size][j] = CR_i[i][j];
			}
			SS_FF[SS_size]        = FF[i];
			tt_CR[SS_size]        = CR[i];
			SS_reward[SS_size]    = rr;
			SS_size++;
			/*------------------------------------------------*/

			/************************************************************************/
			/* Save the inferior parent into the archive                            */
			/************************************************************************/
			if (m_strategy==3 || m_strategy==4)
			{
				if (archive_size < pop_size)
				{
					archive_pop[archive_size] = parent_pop[i];
					archive_size++;
				}
				else
				{
					archive_size = pop_size;
					
					int ii = m_rnd.rndint(0, archive_size-1);
					archive_pop[ii] = parent_pop[i];
				}
			}
			/************************************************************************/

			// update the parent_pop with the better offspring
			parent_pop[i] = child_pop[i];
		}
	}

	/* Step 4: Update mu_CR and mu_F based on the successful CRs and Fs */
	if (SS_size != 0) 
	{
		// update the mu_CR[j] values
		double mean_cr  = 0.0;
		for (i=0;i<SS_size;i++)
		{
			mean_cr += tt_CR[i];
		}
		mean_cr    = mean_cr/((double)SS_size);
		JADE_mu_CR = (1-JADE_c) * JADE_mu_CR + JADE_c * mean_cr;

		// update the mu_F value
		double mean_ff = 0.0;
		double t1      = 0.0;
		double t2      = 0.0;
		for (i=0;i<SS_size;i++)
		{
			t1 += SS_FF[i]*SS_FF[i];
			t2 += SS_FF[i];
		}					
		mean_ff    = t1/t2;					// Lehmer mean
		JADE_mu_FF = (1-JADE_c)*JADE_mu_FF + JADE_c*mean_ff;
	}
}

void CDE_method::run_NoDE_JADE()
{
	int    i, j, j_rand;
	int    r1, r2, r3;
	double low, up;

	double FF[population_size];
	double CR[population_size];

	/************************************************************************/
	/* Calculate the window size and rankings in NoDE                       */
	/************************************************************************/
	candidate_size   = 4;
	NoDE_window_size = estimate_current_window_size(parent_pop, pop_size);
	calculate_ranking_UOPs(parent_pop, pop_size, NoDE_ranks);

	/* Step 1: Initialize the JADE-related parameters */
	int    archive_index;	// if r3 is chosen from archive archive_index=1, else =0
	int    p_index = -1;
	if (gen == 1)
	{
		JADE_mu_CR  = 0.5;
		JADE_mu_FF  = 0.5;
		JADE_c      = 0.1;

		JADE_p       = 0.05;
		archive_size = 0;
	}
	/*------------------------------------------------*/

	sort_pop(parent_pop, pop_size);

	/* Generate the CR and F values */
	for (i=0;i<pop_size;i++)
	{
		// generate F value for each individual (individual level)
		do {
			FF[i] = m_rnd.cauchy(JADE_mu_FF, 0.1);
		} while (FF[i] <= 0.0);
		if (FF[i] > 1.0)	FF[i] = 1.0;

		// generate CR value for each dimension of each individual (component level)
		CR[i] = m_rnd.gaussian(JADE_mu_CR, 0.1);
		if (CR[i] < 0.0)	CR[i] = 0.0;
		if (CR[i] > 1.0)	CR[i] = 1.0;
	}

	for (i=0;i<pop_size;i++)
	{
		m_F = FF[i];

		// strategy 1 ==================================================================
		// randomly choose the p_best individual
		p_index = m_rnd.rndint(0, int(JADE_p*pop_size)-1);
		p_index = sort_index[p_index];

		do {
			r1 = m_rnd.rndint(0, pop_size-1);
		} while (r1 == i);
		do {
			r2 = m_rnd.rndint(0, pop_size-1);
		} while(r2 == i || r2 == r1);
		do { 
			r3 = m_rnd.rndint(0, pop_size-1);
		} while(r3 == i || r3 == r2 || r3 == r1);

		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];
			
			// DE/current-to-pbest mutation
			child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
				m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
				m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);

			if (child_pop[i].xreal[j] < low || child_pop[i].xreal[j] > up)
			{
				child_pop[i].xreal[j] = m_rnd.rndreal(low, up);
			}

			candidate_x[0][j] = child_pop[i].xreal[j];
			update_tmp_boundary(candidate_x[0][j], j);
		}

		// strategy 2 ==================================================================
		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];
			
			// DE/rand-to-pbest mutation
			child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
				m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
				m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);

			if (child_pop[i].xreal[j] < low || child_pop[i].xreal[j] > up)
			{
				child_pop[i].xreal[j] = m_rnd.rndreal(low, up);
			}

			candidate_x[1][j] = child_pop[i].xreal[j];
			update_tmp_boundary(candidate_x[1][j], j);
		}
		
		// strategy 3 ==================================================================
		// choose r3 from the union of parent population and the archive
		archive_index = 0;
		int k = pop_size+archive_size;
		r3 = m_rnd.rndint(0, k-1);
		if (r3>=0 && r3<pop_size)
		{// r3 is chosen from the parent population
			while(r3 == r2 || r3 == r1 || r3 == i)
			{
				r3 = m_rnd.rndint(0, pop_size-1);
			}
		}
		else
		{// r3 is chosen from the archive
			archive_index = 1;
			r3 = r3-pop_size;
		}

		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];
			
			// DE/current-to-pbest mutation with archive
			if (archive_index==0)
			{
				child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
					m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
					m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
			}
			else
			{
				child_pop[i].xreal[j] = parent_pop[i].xreal[j] +
					m_F*(parent_pop[p_index].xreal[j]-parent_pop[i].xreal[j]) +
					m_F*(parent_pop[r2].xreal[j]-archive_pop[r3].xreal[j]);
			}

			if (child_pop[i].xreal[j] < low || child_pop[i].xreal[j] > up)
			{
				child_pop[i].xreal[j] = m_rnd.rndreal(low, up);
			}

			candidate_x[2][j] = child_pop[i].xreal[j];
			update_tmp_boundary(candidate_x[2][j], j);
		}

		// strategy 4 ==================================================================
		for (j=0;j<N_of_x;j++)
		{
			low = lower_bounds[j];
			up  = upper_bounds[j];
			
			// DE/rand-to-pbest mutation with archive
			if (archive_index==0)
			{
				child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
					m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
					m_F*(parent_pop[r2].xreal[j]-parent_pop[r3].xreal[j]);
			}
			else
			{
				child_pop[i].xreal[j] = parent_pop[r1].xreal[j] +
					m_F*(parent_pop[p_index].xreal[j]-parent_pop[r1].xreal[j]) +
					m_F*(parent_pop[r2].xreal[j]-archive_pop[r3].xreal[j]);
			}

			if (child_pop[i].xreal[j] < low || child_pop[i].xreal[j] > up)
			{
				child_pop[i].xreal[j] = m_rnd.rndreal(low, up);
			}

			candidate_x[3][j] = child_pop[i].xreal[j];
			update_tmp_boundary(candidate_x[3][j], j);
		}

		/************************************************************************/
		/* Offspring filtering with the NoDE method                             */
		/************************************************************************/
		strategy_index[i] = offspring_fitering(i, parent_pop, pop_size, child_pop[i]);

		// DE crossover is performed herein 
		j_rand = m_rnd.rndint(0, N_of_x-1);	
		for (j=0; j<N_of_x; j++)
		{
			if (m_rnd.rndreal(0,1)<CR[i] || j==j_rand)
			{
				child_pop[i].xreal[j] = candidate_x[strategy_index[i]-1][j];//child_pop[i].xreal[j];
			}
			else
			{
				child_pop[i].xreal[j] = parent_pop[i].xreal[j];
			}
		}
	}

	evaluate_pop(child_pop, pop_size);

	double SS_FF[population_size], SS_CR[population_size];
	int    SS_size = 0;						// The successful size of S_CR and S_F
	for (i=0;i<pop_size;i++)
	{
		int flag = compare_ind(&child_pop[i], &parent_pop[i]);
		if (flag != -1)
		{
			/* Step 3: Update the accepting flag and store the successful CR and F values */
			SS_FF[SS_size] = FF[i];
			SS_CR[SS_size] = CR[i];
			SS_size++;
			/*------------------------------------------------*/

			/************************************************************************/
			/* Save the inferior parent into the archive                            */
			/************************************************************************/
			if (1)
			{
				if (archive_size < pop_size)
				{
					archive_pop[archive_size] = parent_pop[i];
					archive_size++;
				}
				else
				{
					archive_size = pop_size;
					
					int ii = m_rnd.rndint(0, archive_size-1);
					archive_pop[ii] = parent_pop[i];
				}
			}
			/************************************************************************/

			// update the parent_pop with the better offspring
			parent_pop[i] = child_pop[i];
		}
	}

	/* Step 4: Update mu_CR and mu_F based on the successful CRs and Fs */
	if (SS_size != 0) 
	{
		// update the mu_CR[j] values
		double mean_cr  = 0.0;
		double t1      = 0.0;
		double t2      = 0.0;
		for (i=0;i<SS_size;i++)
		{
			mean_cr += SS_CR[i];
		}
		mean_cr    = mean_cr/((double)SS_size);
		JADE_mu_CR = (1-JADE_c) * JADE_mu_CR + JADE_c * mean_cr;

		// update the mu_F value
		double mean_ff = 0.0;
		t1			   = 0.0;
		t2			   = 0.0;
		for (i=0;i<SS_size;i++)
		{
			t1 += SS_FF[i]*SS_FF[i];
			t2 += SS_FF[i];
		}					
		mean_ff    = t1/t2;					// Lehmer mean
		JADE_mu_FF = (1-JADE_c)*JADE_mu_FF + JADE_c*mean_ff;
	}
}








