// DE_method.h: interface for the CDE_method class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DE_METHOD_H__DD4356E1_7F38_48E8_884D_BD72B64F4530__INCLUDED_)
#define AFX_DE_METHOD_H__DD4356E1_7F38_48E8_884D_BD72B64F4530__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "./EACommonClass/Individual.h"
#include "./EACommonClass/Rand.h"
#include "./EACommonClass/ProblemDef.h"


#include <direct.h>				// include the mkdir() for folder creation

#define out_internal	30		// Ƶ

/* Ӳ̣ԲҪһֽһֽڸ */
#define SIZEOFBUFFER	256*1024L /* СĬΪ256KB */



class CDE_method  
{
public:
	CDE_method();
	virtual ~CDE_method();

	/* ------------------------class methods------------------------ */
public:
	// public methods
	/* initialize the decision variables of different test problems */
	void init_variables();							// initialize the decision variables
	
	void init_normal_variables();					// initialize the normal functions' variables

	/* initialize the population */
	void init_pop();								// initialize the population
	void init_pop_random();							// initialize the population randomly

	/* evaluate the individual and population */
	void evaluate_ind(CIndividual &);				// evaluate the individual

	void evaluate_normal_ind(CIndividual &);		// evaluate the individual of normal functions

	void evaluate_pop(CIndividual *, int);			// evaluate the population

	/* execute the algorithms */
	void Run_Optimizer(int, int, double, int, int);		// execute the optimizer

	/* compare the two individuals */
	int  compare_ind(CIndividual *, CIndividual *);	// compare the two individuals
	void find_best_index(CIndividual *, int);		// find the index of the best individual in the population

	void restart_pop(CIndividual *, int);			// restart the population
	void sort_pop(CIndividual *, int);				// sort the population w.r.t fitness with ascend order

	/* some complementary methods */
	void random_index(int *array_index, int all_size, int size);

	void shell_sort_pop(CIndividual* ,int *, int);		// shell sort routine in ascending order
	void shell_sort_array(double *, int *, int);
	void shell_sort_array_t(tFitness *, int *, int);

	inline  int round(double x) { return int(x>0.0?(x+0.5):(x-0.5)); } // round function

	double median(double *array, int size);			// get the median value of the specific array
	double mean_std(double *array, int size, double &stdv);	// get the mean and standard values of the specific array
	tFitness mean_std_t(tFitness *array, int size, tFitness &stdv);
	void   statistic_pop_fitness(CIndividual *pop, int size);

	void display_result(long int);						// display the results in the screen
	void report_result(long int, ofstream &);			// report the results in the files
	void report_diversity(long int, ofstream &);			// report the diversity in the files
	void report_parameter(long int gen, ofstream &file);

	/************************************************************************/
	/* Methods and attributes for batch execution (ִг)      */
	/************************************************************************/
public:
	void	create_folders(int method_ii, int strategy_ii);		// create the folders
	void	copy_files(char *source, char *target);				// copy files from the source directory to the target directory
	long	filesize(FILE *stream);
	int		copyfile(const char* src,const char* dest);

	/* ------------------------class members------------------------ */	
public:
	// public attributes
	CIndividual parent_pop[2*population_size];				// the main population
	CIndividual child_pop[2*population_size];					// the child population
	CIndividual child_pop1[2*population_size];				// the child population used in BBO_DE
	CIndividual best_individual;			// the current best individual

	CIndividual	mutant_vector;				// save the mutant vector
	CIndividual new_child;					// the new child generated by the genetic operator
	
	int pop_size;							// size of the main population
	int best_index;							// index of the current best individual
	int worst_index;

	int flag_precision;						// stop criterion

	long int max_NFFEs;						// maximal number of fitness function evaluations
	long int max_iterations;				// maximal number of iterations
	long int evaluations;					// current number of fitness function evaluations
	long int gen;							// current generation counter
	int MINIMIZE;

	double upper_bounds[N_of_x];					// upper bounds of the decision variables
	double lower_bounds[N_of_x];					// lower bounds of the decision variables
	double known_optimal;					// known optima of the problems

	double pop_diversity;
	double pop_diversity0;
	double max_interval;

	/* some other class objectives */
	double rnd_seed;
	CRand  m_rnd;

	int func_flag;								// flag of the function optimized
	CProblemDef				m_func;				// normal test functions

	int method_flag;

	int	sort_index[2*population_size];							// only for sorting the population

	/************************************************************************/
	/* Methods and attributes for DE algorithm                              */
	/************************************************************************/
public:
	double	m_F;									// scale factor
	double	m_CR;									// probability of DE crossover
	int		m_strategy;								// mutation strategy

	/************************************************************************/
	/* Methods and attributes for Nonparametric Density Estimation          */
	/************************************************************************/
public:
	double	tmp_lower_bounder[2000], tmp_upper_bounder[2000];
	int		candidate_size;
	double	candidate_x[100][2000];												// candidate solutions
	double	NoDE_window_size;													// window size
	double	NoDE_w[N_of_x];														// window size for each variable  (product kernel)
	int		NoDE_ranks[population_size];										// save the rankings of solutions

	int		neighborhood_index[population_size];
	int		neighborhood_size;
	double	p_radius;
	int		radius;									// the radius of the neighborhood
	int		radius_index;

	void	GetNeighborhoodIndex(int);				// get the neighborhood indices

	int		update_tmp_boundary(const double x, const int j);
	double	Euc_dist(double *x, double *y, int size);
	double	infinity_norm_dist(double *x, double *y, int size);
	double	NoDE_Gaussian_kernel(double u);
	double	estimate_current_window_size(CIndividual *pop, int size);			// estimate the window size of the current population
	void	calculate_ranking_UOPs(CIndividual *pop, int size, int *ranks);		// calculate the rankings of the solutions for unconstrained optimization problems (UOPs)
	double	NDE_of_solution(int baseindex, CIndividual *pop, int size, double *xreal);			// nonparametric density estimation of the solution
	int		offspring_fitering(int baseindex, CIndividual *pop, int size, CIndividual &child);	// offspring filtering based on the probabilities estimated

	int		strategy_index[population_size];
	int		init_strategy, stagnation_counter;
	tFitness prev_best_fitness, best_fitness;

	void	calculate_weighted_center(CIndividual *pop, int size);
	double	weighted_center[N_of_x];


	/*******************************************************************************/
	/* Methods and attributes for CoDE method                                      */
	/*******************************************************************************/
public:
	double	CoDE_CR_F[3][2];						// parameters pool in CoDE [CR, F] = [0.1, 0.1; 0.9, 0.1; 0.2, 0.8]

	CIndividual CoDE_indv[3];						// three temporary individuals generated by CoDE

	void	run_CoDE_method();

	void	run_NoDE_CoDE();						// CoDE with the nonparametric density estimation

	/************************************************************************/
	/* Methods and attributes for JADE methods                              */
	/************************************************************************/
public:	
	double	JADE_mu_CR;
	double	JADE_mu_FF;
	double	JADE_c;
	double	JADE_std_CR;							// standard of generating CR, default=0.1
	double	JADE_std_FF;							// standard of generating F, default=0.1
	double	JADE_S_FF[population_size];				// save the successful F
	double	JADE_S_CR[population_size];				// save the successful CR

	double		JADE_p;								// the control parameter of p
	int         archive_size;						// the archive size of the archive
	CIndividual archive_pop[population_size];		// the archive of JAVE

	void	run_JADE_method();							// DE only with JADE parameter adaptation technique

	void	run_NoDE_JADE();
};

#endif // !defined(AFX_DE_METHOD_H__DD4356E1_7F38_48E8_884D_BD72B64F4530__INCLUDED_)
